﻿using UnityEngine;
using System.Collections;

public class Blob : MonoBehaviour {

    private float m_hAcceleration = 40.0f;
    private float m_hDeceleration = 10.0f;
    private float m_hSpeed = 10.0f;

    private float m_jumpSpeed = 20.0f;
    private float m_jumpIntervalDuration = 0.5f;

    private float m_gravity = 9.81f;
    private float m_gravityFactor = 7.0f;

    private float m_direction = 0.0f;
    public bool m_grounded = false;

    private int m_health = 3;
    private float m_recoveryDuration = 0.3f;

    private Alarm m_jumpPause = new Alarm( );
    private Alarm m_recovery = new Alarm( );

    public  Vector2 m_currentSpeed = Vector2.zero;
    public Transform m_tom = null;
    private Transform m_root = null;
    private Tom m_tomComponent = null;
    private bool m_dead = false;
    private int m_flash = 0;
    private Renderer m_renderer = null;
    public Material m_material = null;
    public GameObject m_ps = null;
    public bool m_firstTime = true;

    void Start( ) {

        m_root = transform.parent;
        m_tomComponent = m_tom.GetComponentInChildren<Tom>( );
        m_renderer = GetComponent<Renderer>( );
        m_jumpPause.start( 1.0f );
        m_health = Random.Range( 1, 4 );

        float scale = 0.75f;

        if ( m_health == 2 ) {
            scale = 1.25f;
        } else if ( m_health == 3 ) {
            scale = 2.0f;
        }

        transform.localScale = new Vector3( scale, scale, 1.0f );
        float smallChange = Random.Range( -0.1f, 0.1f );
        transform.localPosition = new Vector3( 0.0f, scale * 0.5f + smallChange, smallChange );
    }

    void Update( ) {

        if ( m_recovery.isRunning( ) ) {

            m_flash = ( m_flash + 1 ) % 10;

            if ( m_flash > 4 ) {
                m_renderer.enabled = false;
            } else {
                m_renderer.enabled = true;
            }
        } else {
            m_renderer.enabled = true;
        }


        m_jumpPause.step( );
        m_recovery.step( );

        if ( !m_dead ) {

            if ( m_grounded && m_jumpPause.isFinished( ) ) {
                m_direction = Mathf.Sign( m_tom.position.x - m_root.position.x );
                m_currentSpeed.y = m_jumpSpeed * ( ( transform.localScale.x == 2.0f ) ? 1.75f : transform.localScale.x );
                m_grounded = false;
            }

            if ( !m_jumpPause.isRunning( ) ) {

                m_currentSpeed.x += m_direction * m_hAcceleration * Time.deltaTime;

                if ( Mathf.Abs( m_currentSpeed.x ) > m_hSpeed ) {
                    m_currentSpeed.x = Mathf.Sign( m_currentSpeed.x ) * m_hSpeed;
                }
            }

        } else {

            if ( m_firstTime ) {
                m_material = m_renderer.material;
                float factor = Random.Range( 0.25f, 0.75f );
                m_material.color = factor * Color.red + ( 1.0f - factor ) * Color.blue;
                m_firstTime = false;
            }

            float direction = Mathf.Sign( m_currentSpeed.x );
            m_currentSpeed.x += m_hDeceleration * -direction * Time.deltaTime;

            if ( Mathf.Sign( m_currentSpeed.x ) != direction ) {
                m_currentSpeed.x = 0.0f;
            }
        }

        m_currentSpeed.y += -m_gravity * m_gravityFactor * Time.deltaTime;
        m_root.position += new Vector3( m_currentSpeed.x, m_currentSpeed.y ) * Time.deltaTime;

        if ( m_root.position.y < 0.0f ) {

            m_root.position = new Vector3( m_root.position.x, 0.0f );
            m_grounded = true;

            if ( !m_dead ) {
                m_currentSpeed.x = 0;
            }

            m_currentSpeed.y = 0;

            if ( !m_jumpPause.isRunning( ) ) {
                m_jumpPause.start( m_jumpIntervalDuration );
            }
        }

        if ( m_root.position.x < -8.0f ) {
            m_root.position = new Vector3( -8.0f, m_root.position.y );
        }

        if ( m_root.position.x > 8.0f ) {
            m_root.position = new Vector3( 8.0f, m_root.position.y );
        }

        if ( !m_dead || !m_grounded ) {

            if ( !m_recovery.isRunning( ) ) {

                AttackInfo info = m_tomComponent.getAttackInfo( );

                if ( info.type != AttackType.None ) {

                    Bounds self = new Bounds( transform.position, transform.localScale );

                    if ( info.bounds.Intersects( self ) ) {

                        m_recovery.start( m_recoveryDuration );
                        m_jumpPause.start( m_jumpIntervalDuration );
                        GetComponent<AudioSource>( ).Play( );

                        switch ( info.type ) {

                            case AttackType.Slam: {
                                m_health -= 3;
                                float direction = ( transform.position.x - info.bounds.center.x ) / ( info.bounds.size.x * 0.5f );
                                m_currentSpeed.x = 10.0f * direction;
                                m_currentSpeed.y = 25.0f;
                            }
                            break;

                            case AttackType.Dash: {
                                m_health -= 2;
                                m_currentSpeed.x = info.direction.x * 25.0f;
                                m_currentSpeed.y = 0.0f;
                            } break;

                            case AttackType.Arm: {
                                m_health--;
                                m_currentSpeed.x = info.direction.x * 5.0f;
                                m_currentSpeed.y = 10.0f;
                            }
                            break;

                            case AttackType.Leg: {
                                m_health--;
                                m_currentSpeed.x = info.direction.x * 15.0f;
                                m_currentSpeed.y = 15.0f;
                            }
                            break;
                        }

                        if ( m_health <= 0 ) {
                            m_dead = true;
                            m_ps = Instantiate( m_ps ) as GameObject;
                            m_ps.transform.parent = transform;
                        }
                    }
                }
            }
        }

        if ( m_health <= 0 ) {
            m_ps.transform.position = transform.position + new Vector3( 0.0f, 0.0f, -1.0f );
        }
    }

    public AttackInfo getHitBox( ) {

        AttackInfo info = new AttackInfo( );

        if ( !m_dead && !m_recovery.isRunning( ) ) {
            info.bounds = new Bounds( transform.position, transform.localScale );
            info.type = AttackType.Slam;
        }

        return info;
    }

    void OnDrawGizmos( ) {
        Gizmos.color = Color.red;
        Gizmos.DrawWireCube( transform.position, transform.localScale );
    }
}
