﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Spawner : MonoBehaviour {

    private int m_maxBlob = 25;
    private float m_spawnInterval = 5.0f;
    private float m_increaseIntervalDuration = 30.0f;
    private int m_spawnCount = 1;

    public GameObject m_tomRoot = null;
    public GameObject m_blob = null;

    private Alarm m_spawnAlarm = new Alarm( );
    private Alarm m_increaseSpawn = new Alarm( );
    private List<Blob> m_blobs = new List<Blob>( );

    public Transform m_spawn_1 = null;
    public Transform m_spawn_2 = null;
    public Transform m_spawn_3 = null;

    private Material m_mat1 = null;
    private Material m_mat2 = null;
    private Material m_mat3 = null;

    void Start( ) {
        m_spawnAlarm.start( 0.0f );
        m_increaseSpawn.start( m_increaseIntervalDuration );
        m_mat1 = m_spawn_1.GetComponent<Renderer>( ).material;
        m_mat2 = m_spawn_2.GetComponent<Renderer>( ).material;
        m_mat3 = m_spawn_3.GetComponent<Renderer>( ).material;
    }

    void Update( ) {

        if ( Input.GetKey( KeyCode.Escape ) ) {
            Application.Quit( );
        }

        m_spawnAlarm.step( );
        m_increaseSpawn.step( );

        if ( m_spawnCount < 3 && m_increaseSpawn.isFinished( ) ) {

            m_spawnCount++;
            m_increaseSpawn.start( m_increaseIntervalDuration );

            if ( m_spawnCount == 2 ) {
                m_spawn_2.gameObject.SetActive( true );
            }

            if ( m_spawnCount == 3 ) {
                m_spawn_3.gameObject.SetActive( true );
            }
        }

        float factor = 1.0f - Mathf.Clamp01( m_spawnAlarm.m_age / m_spawnAlarm.m_duration );
        Color c = new Color( 0.0f, 0.545098f, 1.0f );
        m_mat1.color = factor * c;
        m_mat2.color = factor * c;
        m_mat3.color = factor * c;

        if ( m_spawnAlarm.isFinished( ) ) {

            GetComponent<AudioSource>( ).Play( );

            {
                GameObject go = Instantiate( m_blob ) as GameObject;
                go.transform.position = m_spawn_1.position;
                Blob b = go.GetComponentInChildren<Blob>( );
                b.m_tom = m_tomRoot.transform;
                m_blobs.Add( b );
                m_spawn_1.position = new Vector3( Random.Range( -7.0f, 7.0f ), 5.0f );
            }

            if ( m_spawnCount > 1 ) {

                m_spawn_2.gameObject.SetActive( true );
                GameObject go = Instantiate( m_blob ) as GameObject;
                go.transform.position = m_spawn_2.position;
                Blob b = go.GetComponentInChildren<Blob>( );
                b.m_tom = m_tomRoot.transform;
                m_blobs.Add( b );
                m_spawn_2.position = new Vector3( Random.Range( -7.0f, 7.0f ), 5.0f );

            }
            
            if ( m_spawnCount > 2 ) {

                m_spawn_3.gameObject.SetActive( true );
                GameObject go = Instantiate( m_blob ) as GameObject;
                go.transform.position = m_spawn_3.position;
                Blob b = go.GetComponentInChildren<Blob>( );
                b.m_tom = m_tomRoot.transform;
                m_blobs.Add( b );
                m_spawn_3.position = new Vector3( Random.Range( -7.0f, 7.0f ), 5.0f );
            }

            m_spawnAlarm.start( m_spawnInterval );
        }

        if ( m_blobs.Count > m_maxBlob - m_spawnCount ) {

            for ( int i = 0; i < m_spawnCount; i++ ) {
                Destroy( m_blobs[ 0 ].m_material );
                Destroy( m_blobs[ 0 ].gameObject );
                m_blobs.RemoveAt( 0 );
            }
        }
    }

    public List<Blob> getBlobs( ) {
        return m_blobs;
    }
}
